/**
 * Copyright (C) 2009 Future Invent Informationsmanagement GmbH. All rights
 * reserved. <http://www.fuin.org/>
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library. If not, see <http://www.gnu.org/licenses/>.
 */
package org.fuin.jmsmvc4swing.model;

import java.util.List;

import org.fuin.jmsmvc4swing.base.Controller;
import org.fuin.jmsmvc4swing.base.JmsJndiEnvironment;
import org.fuin.jmsmvc4swing.jms.MethodResultSender;
import org.fuin.srcgen4javassist.SgArgument;
import org.fuin.srcgen4javassist.SgClass;
import org.fuin.srcgen4javassist.SgClassPool;
import org.fuin.srcgen4javassist.SgConstructor;
import org.fuin.srcgen4javassist.SgMethod;
import org.fuin.srcgen4javassist.SgUtils;

/**
 * Generates a <code>MethodResultSender</code> implementation.
 * 
 * @param <I>
 *            The controller interface the created class is for.
 */
public final class MethodResultSenderGenerator<I extends Controller> extends
        AbstractMethodGenerator<I, MethodResultSender> {

    private final SgClass listenerClass;

    /**
     * Constructor.
     * 
     * @param pool
     *            Class pool to use.
     * @param model
     *            Controller model.
     * @param basePackage
     *            Base package.
     * @param intfMethod
     *            Method from the interface the class depends on.
     */
    public MethodResultSenderGenerator(final SgClassPool pool, final ControllerModel<I> model,
            final String basePackage, final SgMethod intfMethod) {

        super(pool, model, basePackage, intfMethod, "MethodResultSender");

        final String intfName = model.getControllerInterface().getName();
        final String methodName = intfMethod.getName();
        final String listenerClassName = intfName + "$" + SgUtils.firstCharUpper(methodName)
                + "Listener";
        listenerClass = SgClass.create(getPool(), listenerClassName);

    }

    private void addConstructor(final SgClass clasz) {
        final SgConstructor constructor = new SgConstructor(clasz);
        // Adds the arguments implicitly to the constructor
        new SgArgument(constructor, SgClass.create(getPool(), JmsJndiEnvironment.class), "env");
        new SgArgument(constructor, SgClass.create(getPool(), String.class), "topicName");
        new SgArgument(constructor, SgClass.create(getPool(), String.class), "methodName");
        new SgArgument(constructor, SgClass.LONG, "id");
        constructor.addBodyLine("super(env, topicName, methodName, id);");
        clasz.addConstructor(constructor);
    }

    private void addMethods(final SgClass clasz) {

        final List<SgMethod> listenerMethods = listenerClass.getMethods();
        for (int i = 0; i < listenerMethods.size(); i++) {
            final SgMethod listenerMethod = listenerMethods.get(i);
            final SgMethod method = new SgMethod(clasz, "public", SgClass.VOID, listenerMethod
                    .getName());
            final List<SgArgument> args = listenerMethod.getArguments();
            for (int j = 0; j < args.size(); j++) {
                // Adds the argument implicitly to the method
                new SgArgument(method, args.get(j));
            }
            final String pkg = getDestPackage() + "." + listenerMethod.getNameAsPackage();
            method.addBodyLine("publishResult(new " + pkg + ".Result("
                    + listenerMethod.getCommaSeparatedArgumentNames() + "));");
            clasz.addMethod(method);
        }

    }

    /**
     * Create the model class based on the interface informations.
     * 
     * @return Model class.
     */
    public final SgClass createModelClass() {

        // Create class definition
        final SgClass superClass = SgClass.create(getPool(), MethodResultSender.class);
        final SgClass clasz = new SgClass("public", getDestPackage(), getSimpleDestClass(),
                superClass, false, null);
        clasz.addInterface(listenerClass);

        addConstructor(clasz);
        addMethods(clasz);

        getPool().put(clasz);

        return clasz;
    }

}
